include ht48e30.inc
	
; Timing is OK for 4MHz clocking for other speeds check TRW-2.4G datasheet

; DAT must be LSB, otherwise, code must be modified
DAT		equ	PG.0
DATPORT equ	PG
DATT	equ	PGC.0
SCK		equ	PC.5
CE		equ	PB.3
DR		equ	PB.7
CS		equ	PB.2

; Packet lenght in bytes
TRW_DATA_LEN	equ	24
; Address lenght in bytes
TRW_ADDRESS_LEN equ	5



trwvars	.section 'data'
datas	db ?
bitcntr	db ?
bytcntr	db ?
addrptr	db ?
; MUST BE CONTIGUOUS
txbuffer LABEL BYTE
address	db TRW_ADDRESS_LEN DUP(?)
messge	db TRW_DATA_LEN DUP(?)



trwcode	.section 'code'

; Send  a byte
; Call with data in A
trw_write_byte:
		mov datas,a				; Store data
		mov a,8					; 8 bits
		mov bitcntr,a
		clr DATT				; DATA = output
		nop
twbl1:	rlc datas				; C = MSB (shift bit for output)
		clr SCK					; CLK = 0
		set DAT					; DATA = 1 (guess)
		snz C			; if C, we've guessed right
		clr DAT					; otherwise DATA = 0
		nop
		set SCK					; CLK = 1
		sdz bitcntr
		jmp twbl1
		clr SCK					; CLK = 0
		nop
		set DATT				; DATA = input
		ret

; Receive a byte
; Returns data in A
trw_read_byte:
		clr datas				; DATA = 0
		mov a,8					; 8 bits
		mov bitcntr,a
trbl1:	set SCK					; CLK = 1
		nop
		rrca DATPORT			; read data in C
		rlc datas				; and into register	(MSB first)	
		clr SCK					; CLK = 0
		sdz bitcntr
		jmp trbl1
		mov a,datas
		ret

; Initialize TRW-2.4G module
trw_init:
		set CS					; CS = 1
		clr bytcntr
til1:	mov a,bytcntr
		call getconfig			; get byte from table in A
		call trw_write_byte		; send byte in A
		inc bytcntr
		mov a,18				; 18 bytes
		sub a,bytcntr
		sz ACC
		jmp til1 				; repeat
		clr CS					; CS = 0
		ret

; Set comms channel (Rx=off)
; Call with channel in A
trw_setch:
		mov a,datas				; channel *= 2
		add a,datas
		set CS					; CS = 1
		call trw_write_byte		; send byte in A
		clr CS					; CS = 0
		ret

; Enable Tx
trw_settx:
		clr CE					; CE = 0
		nop
		set CS					; CS = 1
		nop
		clr DATT				; DATA = output
		nop
		clr DAT					; DATA = 0
		nop
		nop
sndbit:	set SCK					; CLK = 1
		nop
		clr SCK					; CLK = 0
		nop
		set DATT				; DATA = input
		nop
		clr CS					; CS = 0
		nop
		set CE					; CE = 1
		ret

; Enable Rx
trw_setrx:
		clr CE					; CE = 0
		nop
		set CS					; CS = 1
		nop
		clr DATT				; DATA = output
		nop
		set DAT					; DATA = 1
		jmp sndbit

; Send a message to the remote end
; Call with address in address buffer and packet in packet buffer
trw_sendpacket:
		call trw_settx
		mov a,(TRW_ADDRESS_LEN+TRW_DATA_LEN)	; # bytes
		mov bytcntr,a			; Init # bytes to put
		mov a,OFFSET txbuffer			; Initialize pointer
		mov MP0,a
tsl1:	mov a,IAR0				; get byte in A
		call trw_write_byte		; send byte in A
		inc MP0				; next byte
		sdz bytcntr			; loop
		jmp tsl1
		clr CE					; CE = 0
		ret


; Get a message from a remote end
; Returns:
;	A=0 => no msg waiting
;	A=0xFF => msg in packet buffer
trw_getpacket:
		snz DR				; packet ?
		ret a,0					; NO
		clr CE					; CE = 0
		mov a,TRW_DATA_LEN		; # bytes
		mov bytcntr,a			; Init # bytes to put
		mov a,OFFSET messge			; Initialize pointer
		mov MP0,a
tgl1:	call trw_read_byte		; get byte in A
		mov IAR0,a				; store byte in A in buffer
		inc MP0				; next byte
		sdz bytcntr		; loop
		jmp tgl1
		ret a,0FFh

; Set remote address
; Call with remote address' index in A
trw_address:
		call addresses			; Get memory address for requested module's address
		mov addrptr,a			; save it
		mov a,TRW_ADDRESS_LEN	; # bytes
		mov bytcntr,a
		mov a,OFFSET address			; Initialize pointer
		mov MP0,A
tal1:	call getaddr			; Get address byte in A
		mov IAR0,a				; store byte in A in buffer
		inc MP0				; next byte
		inc addrptr
		sdz bytcntr			; keep byte count
		jmp tal1				; loop
		ret

		
trwdata	.section at LASTPAGE-0100h 'code'
getaddr: mov a,addrptr
addresses:
		addm a,PCL
		ret a,LOW (addrs1-2)	; 2 bytes offset when adding to PCL 
;		ret a,LOW (addrs2-2)
; ...		
addrs1:	ret a,021h
		ret a,043h
		ret a,065h
		ret a,087h
		ret a,0A9h
; addrs2
; ...
getconfig:
		addm a,PCL
		ret a,08Eh
		ret a,008h
		ret a,01Ch
		ret a,0C0h
		ret a,0C0h
		ret a,0BBh
		ret a,0BBh
		ret a,0BBh
		ret a,0BBh
		ret a,0BBh
		ret a,013h			; LOCAL ADDRESS
		ret a,025h			;
		ret a,046h			;
		ret a,079h			;
		ret a,08Ah			;
		ret a,0A3h
		ret a,04Fh
		ret a,014h

		public trw_sendpacket,trw_getpacket,trw_setrx,trw_setch,trw_init,trw_address
		public messge

		.end
